<?php

namespace Drupal\postoffice_inline_styles\EventSubscriber;

use Drupal\postoffice\Email\ThemeAssetsInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\Mailer\Event\MessageEvent;
use Symfony\Component\Mime\Email;
use Symfony\Component\Mime\Message;
use TijsVerkoyen\CssToInlineStyles\CssToInlineStyles;

/**
 * Message subscriber which is responsible for inlining CSS.
 *
 * Runs with priority -100 in order (after Symfony MessageListener which is
 * responsible for body rendering).
 */
class MessageSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events = [];

    if (class_exists('\TijsVerkoyen\CssToInlineStyles\CssToInlineStyles')) {
      $events[MessageEvent::class] = ['onMessage', -100];
    }

    return $events;
  }

  /**
   * Inline CSS if possible.
   */
  public function onMessage(MessageEvent $event): void {
    $message = $event->getMessage();
    if ($message instanceof Email) {
      $inliner = new CssToInlineStyles();
      $css = $this->cssFromAttachments($message);
      $message->html($inliner->convert($message->getHtmlBody(), $css));
    }
  }

  /**
   * Returns CSS from attached libraries if message has attached assets.
   */
  protected function cssFromAttachments(Message $message): string {
    $chunks = [];

    if ($message instanceof ThemeAssetsInterface) {
      $chunks = array_map(
        fn($asset) => @file_get_contents($asset['data']) ?: "",
        $message->getThemeAssets(),
      );
    }

    return implode("\n", $chunks);
  }

}
