<?php

namespace Drupal\postoffice\BodyRenderer;

use Drupal\Core\Asset\AssetResolverInterface;
use Drupal\Core\Asset\AttachedAssets;
use Drupal\Core\Render\RendererInterface;
use Drupal\postoffice\Email\ThemeAssetsInterface;
use Drupal\postoffice\Email\ThemedEmailInterface;
use Symfony\Component\Mime\BodyRendererInterface;
use Symfony\Component\Mime\Email;
use Symfony\Component\Mime\Message;

/**
 * Symfony mailer body renderer for emails themed using Drupal render arrays.
 */
class ThemedBodyRenderer implements BodyRendererInterface {

  /**
   * The drupal core renderer.
   */
  protected RendererInterface $coreRenderer;

  /**
   * The asset resolver.
   */
  protected AssetResolverInterface $assetResolver;

  /**
   * Constructs a new themed body renderer.
   */
  public function __construct(RendererInterface $coreRenderer, AssetResolverInterface $assetResolver) {
    $this->coreRenderer = $coreRenderer;
    $this->assetResolver = $assetResolver;
  }

  /**
   * {@inheritdoc}
   */
  public function render(Message $message): void {
    if ($message instanceof ThemedEmailInterface && $message instanceof Email) {
      $build = $message->buildThemedEmail();
      $message->html((string) $this->coreRenderer->render($build, TRUE));
      if ($message instanceof ThemeAssetsInterface) {
        $assets = AttachedAssets::createFromRenderArray($build);
        $paths = $this->assetResolver->getCssAssets($assets, FALSE);
        $message->setThemeAssets($paths);
      }
    }
  }

}
