<?php

namespace Drupal\Tests\postoffice\Unit;

use Drupal\Core\Asset\AssetResolverInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\postoffice\BodyRenderer\ThemedBodyRenderer;
use Drupal\postoffice\Email\ThemeAssetsInterface;
use Drupal\postoffice\Email\ThemedEmailInterface;
use Drupal\Tests\UnitTestCase;
use Prophecy\Argument;
use Symfony\Component\Mime\Email;

/**
 * @coversDefaultClass \Drupal\postoffice\BodyRenderer\ThemedBodyRenderer
 * @group postoffice
 */
class ThemedBodyRendererTest extends UnitTestCase {

  public function testRenderSymfonyEmail(): void {
    $email = $this->prophesize(Email::class);
    $coreRenderer = $this->prophesize(RendererInterface::class);
    $assetResolver = $this->prophesize(AssetResolverInterface::class);

    $coreRenderer->render(Argument::any(), Argument::any())->shouldNotBeCalled();
    $email->html(Argument::any())->shouldNotBeCalled();
    $assetResolver->getCssAssets(Argument::any(), Argument::any())->shouldNotBeCalled();

    $renderer = new ThemedBodyRenderer($coreRenderer->reveal(), $assetResolver->reveal());
    $renderer->render($email->reveal());
  }

  public function testRenderThemedEmailWithoutCssAssets(): void {
    $email = $this->prophesize(Email::class)
      ->willImplement(ThemedEmailInterface::class);
    $coreRenderer = $this->prophesize(RendererInterface::class);
    $assetResolver = $this->prophesize(AssetResolverInterface::class);

    $renderArray = ['#theme' => 'hello_world'];
    $markup = '<p>hello world!</p>';

    $email->buildThemedEmail()->willReturn($renderArray);
    $coreRenderer->render($renderArray, TRUE)->willReturn($markup);
    $email->html($markup)->willReturn($email);
    $assetResolver->getCssAssets(Argument::any(), Argument::any())->shouldNotBeCalled();

    $renderer = new ThemedBodyRenderer($coreRenderer->reveal(), $assetResolver->reveal());
    $renderer->render($email->reveal());
  }

  public function testRenderThemedEmail(): void {
    $email = $this->prophesize(Email::class)
      ->willImplement(ThemedEmailInterface::class)
      ->willImplement(ThemeAssetsInterface::class);
    $coreRenderer = $this->prophesize(RendererInterface::class);
    $assetResolver = $this->prophesize(AssetResolverInterface::class);

    $renderArray = [
      '#theme' => 'hello_world',
      '#attached' => [
        'library' => [
          'postoffice/email.test',
        ],
      ],
    ];
    $cssAssets = ['path/to/some/library/email.css'];
    $markup = '<p>hello world!</p>';

    $email->buildThemedEmail()->willReturn($renderArray);
    $coreRenderer->render($renderArray, TRUE)->willReturn($markup);
    $email->html($markup)->willReturn($email);
    $assetResolver->getCssAssets(Argument::any(), FALSE)->willReturn($cssAssets);
    $email->setThemeAssets($cssAssets)->shouldBeCalled();

    $renderer = new ThemedBodyRenderer($coreRenderer->reveal(), $assetResolver->reveal());
    $renderer->render($email->reveal());
  }

}
